/**
  ******************************************************************************
  * @author  Waveshare Team
  ******************************************************************************
**/
#include "IT8951.h"
#include <string.h>
#include "software_spi.h"
#include "SD.h"

void IT8951LoadImgStart(IT8951LdImgInfo* pstLdImgInfo);
void IT8951LoadImgAreaStart(IT8951LdImgInfo* pstLdImgInfo ,IT8951AreaImgInfo* pstAreaImgInfo);
void IT8951HostAreaPackedPixelWrite(IT8951LdImgInfo* pstLdImgInfo,IT8951AreaImgInfo* pstAreaImgInfo);
void IT8951DisplayArea1bpp(TWord usX, TWord usY, TWord usW, TWord usH, TWord usDpyMode, TByte ucBGGrayVal, TByte ucFGGrayVal);
void IT8951Load1bppImage(TByte* p1bppImgBuf, TWord usX, TWord usY, TWord usW, TWord usH);
void LCDWriteCmdCode(TWord usCmdCode);
void LCDWriteData(TWord usData);
void LCDWriteNData(TWord* pwBuf, TDWord ulSizeWordCnt);
TWord IT8951ReadReg(TWord usRegAddr);
TWord LCDReadData();
void LCDReadNData(TWord* pwBuf, TDWord ulSizeWordCnt);
void LCDSendCmdArg(TWord usCmdCode,TWord* pArg, TWord usNumArg);
void IT8951WriteReg(TWord usRegAddr,TWord usValue);
void IT8951MemBurstReadTrigger(TDWord ulMemAddr , TDWord ulReadSize);
void IT8951MemBurstWrite(TDWord ulMemAddr , TDWord ulWriteSize);
void IT8951MemBurstWriteProc(TDWord ulMemAddr , TDWord ulWriteSize, TWord* pSrcBuf );
void IT8951MemBurstReadProc(TDWord ulMemAddr , TDWord ulReadSize, TWord* pDestBuf );
void IT8951SetImgBufBaseAddr(TDWord ulImgBufAddr);
void IT8951DisplayArea(TWord usX, TWord usY, TWord usW, TWord usH, TWord usDpyMode);
void IT8951DisplayAreaBuf(TWord usX, TWord usY, TWord usW, TWord usH, TWord usDpyMode, TDWord ulDpyBufAddr);
void IT8951HostAreaPackedPixelWriteFill(IT8951LdImgInfo* pstLdImgInfo,IT8951AreaImgInfo* pstAreaImgInfo, TWord pixel);
void IT8951HostAreaPackedPixelWriteFromFile(IT8951LdImgInfo* pstLdImgInfo,IT8951AreaImgInfo* pstAreaImgInfo, File bmpFile);

#if defined (IT8951_Interface_I80)
void delay(void)
{
  TWord i,j;
  for (i=0;i<10000;i++)
    for (j=0;j<10;j++);
}

void delay_nop(void)
{
  TWord i;
  for (i=0;i<1000;i++); 
}

void InterfaceInit(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  
  delay();
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB | RCC_AHB1Periph_GPIOC, ENABLE);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3 | GPIO_Pin_6 | GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  SET_RESET; 
  SET_WEN; 
  SET_RD; 
  SET_CS; 
  SET_DC; 
  WRITE_DATA(0xffff);
  CLR_RESET;
  delay();
  SET_RESET;
  delay();
}

void GPIO_Configuration_Out(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
}

void GPIO_Configuration_In(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
}

void LCDWaitForReady(void)
{
  TByte ulData = READ_RDY;
  while(ulData == 0)
  {
    ulData = READ_RDY;
    
  }
}

void gpio_i80_16b_cmd_out(TWord usCmd)
{
  GPIO_Configuration_Out();

  LCDWaitForReady();

  CLR_DC;
  CLR_CS;
  CLR_WEN;
  WRITE_DATA(usCmd);
  SET_WEN;
  SET_CS;
}

//-------------------------------------------------------------------
//Host controller Write Data for 16 bits using GPIO simulation
//-------------------------------------------------------------------
void gpio_i80_16b_data_out(TWord usData)
{
  GPIO_Configuration_Out();

  LCDWaitForReady();

  SET_DC;
  CLR_CS;
  CLR_WEN;
  WRITE_DATA(usData);  
  SET_WEN; 
  SET_CS; 
}
//-------------------------------------------------------------------
//Host controller Read Data for 16 bits using GPIO simulation
//-------------------------------------------------------------------
TWord gpio_i80_16b_data_in(void)
{
  TWord usData;

  GPIO_Configuration_In();

  LCDWaitForReady();

  SET_DC;
  CLR_CS;
  CLR_RD;
  delay_nop();
  usData = READ_DATA;
  SET_RD;
  SET_CS;

  return usData;
}

//-----------------------------------------------------------------
//Host controller function 2 ¡V Write command code to host data Bus
//-----------------------------------------------------------------
void LCDWriteCmdCode(TWord usCmdCode)
{
  //wait for ready
  LCDWaitForReady();
  //write cmd code
  gpio_i80_16b_cmd_out(usCmdCode);
}

//-----------------------------------------------------------
//Host controller function 3 ¡V Write Data to host data Bus
//-----------------------------------------------------------
void LCDWriteData(TWord usData)
{
  //wait for ready
  LCDWaitForReady();
  //write data
  gpio_i80_16b_data_out(usData);
}

//-----------------------------------------------------------
//Host controller function 4 ¡V Read Data from host data Bus
//-----------------------------------------------------------
TWord LCDReadData(void)
{
  TWord usData;
  //wait for ready
  LCDWaitForReady();
  //read data from host data bus
  usData = gpio_i80_16b_data_in();
  return usData;
}

#endif

#if defined (IT8951_Interface_SPI)
void delayMs()
{
  TWord i,j;
  for (i=0;i<10000;i++)
    for (j=0;j<10;j++);
}

void delay_nop()
{
  TWord i;
  for (i=0;i<10000;i++); 
}

#define SPI_Enable                          digitalWrite(pinCS, LOW) //dont need for this softSPI
#define SPI_Disable                         digitalWrite(pinCS, HIGH) //dont need for this softSPI

void SPI_Configuration(void)
{
  spi_setup();
}

void SPI_Send_byte(uint16_t data)
{
  spi_transfer(data);
}

uint16_t SPI_Receive_byte(void)
{
  return spi_transfer(0x00);
}

void InterfaceInit()
{
  SPI_Configuration();

  pinMode(pinCS, OUTPUT);
  pinMode(pinREAD_RDY, INPUT);
      
  SPI_Disable;

  pinMode(pinRESET_INIT, OUTPUT);
  
  digitalWrite(pinRESET_INIT, LOW);
  delayMs();
  digitalWrite(pinRESET_INIT, HIGH);
  delayMs();
}

void LCDWaitForReady()
{
  TByte ulData = READ_RDY;
  while(ulData == 0)
  {
    ulData = READ_RDY;
  }
}

//-----------------------------------------------------------------
//Host controller function 2   Write command code to host data Bus
//-----------------------------------------------------------------
void LCDWriteCmdCode(TWord usCmdCode)
{
  //Set Preamble for Write Command
  TWord wPreamble = 0x6000; 

  SPI_Enable;
  
  LCDWaitForReady();
  SPI_Send_byte(wPreamble);   
  
  LCDWaitForReady();
  SPI_Send_byte(usCmdCode);     
  
  delay_nop();
  
  SPI_Disable;
}

//-----------------------------------------------------------
//Host controller function 3    Write Data to host data Bus
//-----------------------------------------------------------
void LCDWriteData(TWord usData)//ok
{
  TWord wPreamble = 0x0000;

  SPI_Enable;
  
  LCDWaitForReady();
  SPI_Send_byte(wPreamble);   
  
  LCDWaitForReady();
  SPI_Send_byte(usData);  
  
  delay_nop();
  
  SPI_Disable;
}

void LCDWriteNData(TWord* pwBuf, TDWord ulSizeWordCnt)
{
  TWord wPreamble = 0x0000;
  TWord i;

  SPI_Enable;
  
  LCDWaitForReady();
  SPI_Send_byte(wPreamble);   
  
  LCDWaitForReady();
  //Send Data
  for(i=0;i<ulSizeWordCnt;i++)
  {
      SPI_Send_byte(pwBuf[i]);
  }
  
  delay_nop();
  
  SPI_Disable;
}  

//-----------------------------------------------------------
//Host controller function 4   Read Data from host data Bus
//-----------------------------------------------------------
TWord LCDReadData()
{
  TWord wRData; 

  //set type and direction
  TWord wPreamble = 0x1000;
  
  SPI_Enable;
  
  LCDWaitForReady();
  SPI_Send_byte(wPreamble);   
  
  LCDWaitForReady();
  SPI_Receive_byte();//Dummy
  
  LCDWaitForReady();
  //Read Data
  wRData = SPI_Receive_byte();
  
  SPI_Disable;
  
  return wRData;
}

//-------------------------------------------------------------------
//  Read Burst N words Data
//-------------------------------------------------------------------
void LCDReadNData(TWord* pwBuf, TDWord ulSizeWordCnt)
{
  TDWord i;
  
  //set type and direction
  TWord wPreamble = 0x1000;

  SPI_Enable;
  
  //Send Preamble before reading data
  LCDWaitForReady();
  SPI_Send_byte(wPreamble);   

  LCDWaitForReady();
  SPI_Receive_byte();//Dummy
  
  LCDWaitForReady();
  for(i=0;i<ulSizeWordCnt;i++)
  {
    //Read Data
    pwBuf[i]= SPI_Receive_byte();
  }
  
  SPI_Disable;
}

#endif

#if defined (IT8951_Interface_I2C)
   
#define SET_SDA()     GPIO_SetBits(CT_SDA_GPIO_PORT,CT_SDA_GPIO_PIN)    
#define CLR_SDA()     GPIO_ResetBits(CT_SDA_GPIO_PORT,CT_SDA_GPIO_PIN)  

#define SET_CLK()     GPIO_SetBits(CT_SCL_GPIO_PORT,CT_SCL_GPIO_PIN)    
#define CLR_CLK()     GPIO_ResetBits(CT_SCL_GPIO_PORT,CT_SCL_GPIO_PIN)

#define CT_READ_SDA   GPIO_ReadInputDataBit(CT_SDA_GPIO_PORT,CT_SDA_GPIO_PIN)
#define CT_IIC_SDA     

#define CT_SDA_CLK         RCC_AHB1Periph_GPIOB
#define CT_SDA_GPIO_PORT   GPIOB
#define CT_SDA_GPIO_PIN    GPIO_Pin_7

#define CT_SCL_CLK         RCC_AHB1Periph_GPIOB
#define CT_SCL_GPIO_PORT   GPIOB
#define CT_SCL_GPIO_PIN    GPIO_Pin_6

void InterfaceInit()
{
  GPIO_InitTypeDef GPIO_InitStructure;
  
  RCC_AHB1PeriphClockCmd(CT_SDA_CLK | CT_SCL_CLK | RCC_AHB1Periph_GPIOC, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);


  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  
  GPIO_InitStructure.GPIO_Pin = CT_SCL_GPIO_PIN;
  GPIO_Init(CT_SCL_GPIO_PORT, &GPIO_InitStructure); 
  
  CLR_CLK();
  
  GPIO_InitStructure.GPIO_Pin = CT_SDA_GPIO_PIN;
  GPIO_Init(CT_SDA_GPIO_PORT, &GPIO_InitStructure); 
  
  SET_SDA();
  
  delay_ms(100);
  
  SET_CLK();
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
  GPIO_Init(GPIOC, &GPIO_InitStructure);  
  
  GPIO_ResetBits(GPIOC,GPIO_Pin_5);
  delay_ms(100);
  GPIO_SetBits(GPIOC,GPIO_Pin_5);
  delay_ms(100);
}

void delay_nop()
{
  TWord i;
  for (i=0;i<1000;i++); 
}

void CT_SDA_IN()
{
  GPIO_InitTypeDef GPIO_InitStructure;
  RCC_AHB1PeriphClockCmd(CT_SDA_CLK , ENABLE);
  /* Configure PF6 PF7 PF8 PF9 in output pushpull mode */ 
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = CT_SDA_GPIO_PIN;
  GPIO_Init(CT_SDA_GPIO_PORT, &GPIO_InitStructure); 
}

void CT_SDA_OUT()
{
  GPIO_InitTypeDef GPIO_InitStructure;
  RCC_AHB1PeriphClockCmd(CT_SDA_CLK , ENABLE);
  /* Configure PF6 PF7 PF8 PF9 in output pushpull mode */ 
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = CT_SDA_GPIO_PIN;
  GPIO_Init(CT_SDA_GPIO_PORT, &GPIO_InitStructure); 
}

//²úÉúI2CÆðÊ¼ÐÅºÅ
void CT_I2C_Start(void)
{
  CT_SDA_OUT();     //sdaÏßÊä³ö
  SET_SDA();        
  SET_CLK();
  delay_us(1);
  CLR_SDA();//START:when CLK is high,DATA change form high to low 
  delay_us(1);
  CLR_CLK();//Ç¯×¡I2C×ÜÏß£¬×¼±¸·¢ËÍ»ò½ÓÊÕÊý¾Ý 
}   

//²úÉúI2CÍ£Ö¹ÐÅºÅ
void CT_I2C_Stop(void)
{
  CT_SDA_OUT();//sdaÏßÊä³ö
  
  CLR_CLK();
  CLR_SDA();//STOP:when CLK is high DATA change form low to high
  delay_us(1);
  SET_CLK();
  delay_us(1);  
  SET_SDA();//·¢ËÍI2C×ÜÏß½áÊøÐÅºÅ
  delay_us(1);    
}

//µÈ´ýÓ¦´ðÐÅºÅµ½À´
//·µ»ØÖµ£º1£¬½ÓÊÕÓ¦´ðÊ§°Ü
//        0£¬½ÓÊÕÓ¦´ð³É¹¦
u8 CT_I2C_Wait_Ack(void)
{
  u16 ucErrTime=0;
  CT_SDA_IN(); 
  SET_CLK();
  //delay_us(1);  
  while(CT_READ_SDA)
  {
    ucErrTime++;
    if(ucErrTime>250)
    {
      CT_I2C_Stop();
      return 1;
    }
  }
  CLR_CLK();//Ê±ÖÓÊä³ö0      
  return 0;  
} 

//²úÉúACKÓ¦´ð
void CT_I2C_Ack(void)
{
  CLR_CLK();
  CT_SDA_OUT();
  CLR_SDA();
  delay_us(1);
  SET_CLK();
  delay_us(1);
  CLR_CLK();
}

//²»²úÉúACKÓ¦´ð       
void CT_I2C_NAck(void)
{
  CLR_CLK();
  CT_SDA_OUT();
  SET_SDA();
  delay_us(1);
  SET_CLK();
  delay_us(1);
  CLR_CLK();
}   

//I2C·¢ËÍÒ»¸ö×Ö½Ú
//·µ»Ø´Ó»úÓÐÎÞÓ¦´ð
//1£¬ÓÐÓ¦´ð
//0£¬ÎÞÓ¦´ð       
void CT_I2C_Send_Byte(u8 txd)
{                        
  u8 t;   
  CT_SDA_OUT();       
  CLR_CLK();//À­µÍÊ±ÖÓ¿ªÊ¼Êý¾Ý´«Êä
  for(t=0;t<8;t++)
  {              
    if((txd&0x80)>>7)
      SET_SDA();
    else  
      CLR_SDA();

    txd<<=1;  
    delay_us(1);
    SET_CLK();
    delay_us(1);
    CLR_CLK();  
    delay_us(1);
  }  
}       

//¶Á1¸ö×Ö½Ú£¬ack=1Ê±£¬·¢ËÍACK£¬ack=0£¬·¢ËÍnACK   
u8 CT_I2C_Read_Byte(unsigned char ack)
{
  unsigned char i,receive=0;
  CT_SDA_IN();//SDAÉèÖÃÎªÊäÈë
  
  for(i=0;i<8;i++ )
  {
    CLR_CLK();
    delay_us(1);
    SET_CLK();
    receive<<=1;
    if(CT_READ_SDA)receive++;
    delay_us(1);
  }
  
  if (!ack)CT_I2C_NAck();//·¢ËÍnACK
  else CT_I2C_Ack(); //·¢ËÍACK   

  return receive;
}

#define I2C_SLAVE_ID                0x46 //7-bits S1
//#define I2C_SLAVE_ID              0x35 //7-bits S2
#define I80_I2C_CMD_TYPE_CMD        0x00 //Preamble of Command
#define I80_I2C_CMD_TYPE_DATA       0x80 //Preamble of Data

void LCDWaitForReady()
{
  TByte ulData = READ_RDY;
  while(ulData == 0)
  {
    ulData = READ_RDY;
  }
}

void i2c_master_tx(TByte ucSlaveID, TByte Premable, TByte ulSize, TByte* pWBuf)
{
  int i;
  
  CT_I2C_Start();
  
  CT_I2C_Send_Byte(ucSlaveID << 1 | 0);
  CT_I2C_Wait_Ack();
  
  CT_I2C_Send_Byte(Premable);
  CT_I2C_Wait_Ack();    
  
  for(i=0;i<ulSize;i++)
  {
    CT_I2C_Send_Byte(pWBuf[i]);
    CT_I2C_Wait_Ack();  
  } 
  
  CT_I2C_Stop();
}

//------------------------------------------------------
//Pseudo Code - Basic i2c Read function
//------------------------------------------------------
void i2c_master_rx(TByte ucSlaveID, TByte Premable, TByte ulSize, TByte* pRBuf)
{
  int i;
  
  CT_I2C_Start();

  CT_I2C_Send_Byte(ucSlaveID << 1 | 0);
  CT_I2C_Wait_Ack(); 

  CT_I2C_Send_Byte(Premable);
  CT_I2C_Wait_Ack();

  CT_I2C_Start();
  
  CT_I2C_Send_Byte(ucSlaveID << 1 | 1);
  CT_I2C_Wait_Ack();
  
  pRBuf[0] = CT_I2C_Read_Byte(1);
  pRBuf[0] = CT_I2C_Read_Byte(1);

  for(i=0;i<ulSize;i++)
  {
      pRBuf[i] = CT_I2C_Read_Byte(i==(ulSize-1)?0:1);
      delay_nop();
  }

  CT_I2C_Stop();
}

//-----------------------------------------------------------------
//Host controller function 2   Write command code to host data Bus
//-----------------------------------------------------------------
void LCDWriteCmdCode(TWord usCmdCode)
{
  LCDWaitForReady();

  i2c_master_tx(I2C_SLAVE_ID, I80_I2C_CMD_TYPE_CMD, 2, (TByte*)&usCmdCode); 
}

//-----------------------------------------------------------
//Host controller function 3   Write Data to host data Bus
//-----------------------------------------------------------
void LCDWriteData(TWord usData)
{
  LCDWaitForReady();
  
  //Send Preamble and Data
  i2c_master_tx(I2C_SLAVE_ID, I80_I2C_CMD_TYPE_DATA, 2, (TByte*)&usData); 
}

void LCDWriteNData(TWord* pwBuf, TDWord ulSizeWordCnt)
{
  //Send Data
  LCDWaitForReady();
  //Send Preamble and Data
  i2c_master_tx(I2C_SLAVE_ID, I80_I2C_CMD_TYPE_DATA, ulSizeWordCnt*2, (TByte*)pwBuf); 
}  

//-----------------------------------------------------------
//Host controller function 4    Read Data from host data Bus
//-----------------------------------------------------------
TWord LCDReadData()
{
  TWord wRData; 

  LCDWaitForReady();

  //Read 1 16-bits Data
  i2c_master_rx(I2C_SLAVE_ID, I80_I2C_CMD_TYPE_DATA, 2, (TByte*)&wRData); 

  return wRData;
}

//-------------------------------------------------------------------
//  Read Burst N words Data
//-------------------------------------------------------------------
void LCDReadNData(TWord* pwBuf, TDWord ulSizeWordCnt)
{
  LCDWaitForReady();

  i2c_master_rx(I2C_SLAVE_ID, I80_I2C_CMD_TYPE_DATA, ulSizeWordCnt, (TByte *)pwBuf); 
}


#endif

//-----------------------------------------------------------
//Host controller function 5 ¡V Write command to host data Bus with aruments
//-----------------------------------------------------------
void LCDSendCmdArg(TWord usCmdCode,TWord* pArg, TWord usNumArg)
{
     TWord i;
     //Send Cmd code
     LCDWriteCmdCode(usCmdCode);
     //Send Data
     for(i=0;i<usNumArg;i++)
     {
         LCDWriteData(pArg[i]);
     }
}

//-----------------------------------------------------------
//Host Cmd 1 ¡V SYS_RUN
//-----------------------------------------------------------
void IT8951SystemRun()
{
    LCDWriteCmdCode(IT8951_TCON_SYS_RUN);
}

//-----------------------------------------------------------
//Host Cmd 2 - STANDBY
//-----------------------------------------------------------
void IT8951StandBy()
{
    LCDWriteCmdCode(IT8951_TCON_STANDBY);
}

//-----------------------------------------------------------
//Host Cmd 3 - SLEEP
//-----------------------------------------------------------
void IT8951Sleep()
{
    LCDWriteCmdCode(IT8951_TCON_SLEEP);
}

//-----------------------------------------------------------
//Host Cmd 4 - REG_RD
//-----------------------------------------------------------
TWord IT8951ReadReg(TWord usRegAddr)
{
  TWord usData;
  //----------I80 Mode-------------
  //Send Cmd and Register Address
  LCDWriteCmdCode(IT8951_TCON_REG_RD);
  LCDWriteData(usRegAddr);
  //Read data from Host Data bus
  usData = LCDReadData();
  return usData;
}

//-----------------------------------------------------------
//Host Cmd 5 - REG_WR
//-----------------------------------------------------------
void IT8951WriteReg(TWord usRegAddr,TWord usValue)
{
  //I80 Mode
  //Send Cmd , Register Address and Write Value
  LCDWriteCmdCode(IT8951_TCON_REG_WR);
  LCDWriteData(usRegAddr);
  LCDWriteData(usValue);
}

//-----------------------------------------------------------
//Host Cmd 6 - MEM_BST_RD_T
//-----------------------------------------------------------
void IT8951MemBurstReadTrigger(TDWord ulMemAddr , TDWord ulReadSize)
{
    TWord usArg[4];
    //Setting Arguments for Memory Burst Read
    usArg[0] = (TWord)(ulMemAddr & 0x0000FFFF); //addr[15:0]
    usArg[1] = (TWord)( (ulMemAddr >> 16) & 0x0000FFFF ); //addr[25:16]
    usArg[2] = (TWord)(ulReadSize & 0x0000FFFF); //Cnt[15:0]
    usArg[3] = (TWord)( (ulReadSize >> 16) & 0x0000FFFF ); //Cnt[25:16]
    //Send Cmd and Arg
    LCDSendCmdArg(IT8951_TCON_MEM_BST_RD_T , usArg , 4);
}

//-----------------------------------------------------------
//Host Cmd 7 - MEM_BST_RD_S
//-----------------------------------------------------------
void IT8951MemBurstReadStart()
{
    LCDWriteCmdCode(IT8951_TCON_MEM_BST_RD_S);
}

//-----------------------------------------------------------
//Host Cmd 8 - MEM_BST_WR
//-----------------------------------------------------------
void IT8951MemBurstWrite(TDWord ulMemAddr , TDWord ulWriteSize)
{
    TWord usArg[4];
    //Setting Arguments for Memory Burst Write
    usArg[0] = (TWord)(ulMemAddr & 0x0000FFFF); //addr[15:0]
    usArg[1] = (TWord)( (ulMemAddr >> 16) & 0x0000FFFF ); //addr[25:16]
    usArg[2] = (TWord)(ulWriteSize & 0x0000FFFF); //Cnt[15:0]
    usArg[3] = (TWord)( (ulWriteSize >> 16) & 0x0000FFFF ); //Cnt[25:16]
    //Send Cmd and Arg
    LCDSendCmdArg(IT8951_TCON_MEM_BST_WR , usArg , 4);
}

//-----------------------------------------------------------
//Host Cmd 9 - MEM_BST_END
//-----------------------------------------------------------
void IT8951MemBurstEnd(void)
{
    LCDWriteCmdCode(IT8951_TCON_MEM_BST_END);
}

//-----------------------------------------------------------
//Example of Memory Burst Write
//-----------------------------------------------------------
// ****************************************************************************************
// Function name: IT8951MemBurstWriteProc( )
//
// Description:
//   IT8951 Burst Write procedure
//      
// Arguments:
//      TDWord ulMemAddr: IT8951 Memory Target Address
//      TDWord ulWriteSize: Write Size (Unit: Word)
//      TByte* pDestBuf - Buffer of Sent data
// Return Values:
//   NULL.
// Note:
//
// ****************************************************************************************
void IT8951MemBurstWriteProc(TDWord ulMemAddr , TDWord ulWriteSize, TWord* pSrcBuf )
{
    
    TDWord i;
 
    //Send Burst Write Start Cmd and Args
    IT8951MemBurstWrite(ulMemAddr , ulWriteSize);
 
    //Burst Write Data
    for(i=0;i<ulWriteSize;i++)
    {
        LCDWriteData(pSrcBuf[i]);
    }
 
    //Send Burst End Cmd
    IT8951MemBurstEnd();
}

// ****************************************************************************************
// Function name: IT8951MemBurstReadProc( )
//
// Description:
//   IT8951 Burst Read procedure
//      
// Arguments:
//      TDWord ulMemAddr: IT8951 Read Memory Address
//      TDWord ulReadSize: Read Size (Unit: Word)
//      TByte* pDestBuf - Buffer for storing Read data
// Return Values:
//   NULL.
// Note:
//
// ****************************************************************************************
void IT8951MemBurstReadProc(TDWord ulMemAddr , TDWord ulReadSize, TWord* pDestBuf )
{
    #ifndef IT8951_Interface_SPI
    TDWord i;
    #endif
    //Send Burst Read Start Cmd and Args
    IT8951MemBurstReadTrigger(ulMemAddr , ulReadSize);
          
    //Burst Read Fire
    IT8951MemBurstReadStart();
    
    #ifdef IT8951_Interface_SPI
    
    //Burst Read Request for SPI interface only
    LCDReadNData(pDestBuf, ulReadSize);
    
    #else
    //Burst Read Data
    for(i=0;i<ulReadSize;i++)
    {
        pDestBuf[i] = LCDReadData();        
    }
    
    #endif

    //Send Burst End Cmd
    IT8951MemBurstEnd(); //the same with IT8951MemBurstEnd()
}

//-----------------------------------------------------------
//Host Cmd 10 - LD_IMG
//-----------------------------------------------------------
void IT8951LoadImgStart(IT8951LdImgInfo* pstLdImgInfo)
{
    TWord usArg;
    //Setting Argument for Load image start
    usArg = (pstLdImgInfo->usEndianType << 8 )
    |(pstLdImgInfo->usPixelFormat << 4)
    |(pstLdImgInfo->usRotate);
    //Send Cmd
    LCDWriteCmdCode(IT8951_TCON_LD_IMG);
    //Send Arg
    LCDWriteData(usArg);
}

//-----------------------------------------------------------
//Host Cmd 11 - LD_IMG_AREA
//-----------------------------------------------------------
void IT8951LoadImgAreaStart(IT8951LdImgInfo* pstLdImgInfo ,IT8951AreaImgInfo* pstAreaImgInfo)
{
    TWord usArg[5];
    //Setting Argument for Load image start
    usArg[0] = (pstLdImgInfo->usEndianType << 8 )
    |(pstLdImgInfo->usPixelFormat << 4)
    |(pstLdImgInfo->usRotate);
    usArg[1] = pstAreaImgInfo->usX;
    usArg[2] = pstAreaImgInfo->usY;
    usArg[3] = pstAreaImgInfo->usWidth;
    usArg[4] = pstAreaImgInfo->usHeight;
    //Send Cmd and Args
    LCDSendCmdArg(IT8951_TCON_LD_IMG_AREA , usArg , 5);
}

//-----------------------------------------------------------
//Host Cmd 12 - LD_IMG_END
//-----------------------------------------------------------
void IT8951LoadImgEnd(void)
{
    LCDWriteCmdCode(IT8951_TCON_LD_IMG_END);
}

//Global structures and variables
I80IT8951DevInfo gstI80DevInfo;
TByte* gpFrameBuf;
// TByte FrameBuf[59520];
TDWord gulImgBufAddr; //IT8951 Image buffer address

void GetIT8951SystemInfo(void* pBuf)
{
  TWord* pusWord = (TWord*)pBuf;
  I80IT8951DevInfo* pstDevInfo;
  
  #ifdef IT8951_Interface_I80
  TDWord i; 
  #endif

  LCDWriteCmdCode(USDEF_I80_CMD_GET_DEV_INFO);
 
  #if defined(IT8951_Interface_SPI) || defined(IT8951_Interface_I2C)
  //Burst Read Request for SPI interface only
  LCDReadNData(pusWord, sizeof(I80IT8951DevInfo));//Polling HRDY for each words(2-bytes) if possible
  #else
  //I80 interface - Single Read available
  for(i=0;i<sizeof(I80IT8951DevInfo)/2;i++)
  {
      pusWord[i] = LCDReadData();
  }
  #endif
  
  //Show Device information of IT8951
  pstDevInfo = (I80IT8951DevInfo*)pBuf;
  Serial.printf("Panel(W,H) = (%d,%d)\r\n",
  pstDevInfo->usPanelW, pstDevInfo->usPanelH );
  Serial.printf("Image Buffer Address = %X\r\n",
  pstDevInfo->usImgBufAddrL | (pstDevInfo->usImgBufAddrH << 16));
  //Show Firmware and LUT Version
  Serial.printf("FW Version = %s\r\n", (TByte*)pstDevInfo->usFWVersion);
  Serial.printf("LUT Version = %s\r\n", (TByte*)pstDevInfo->usLUTVersion);
}

//-----------------------------------------------------------
//Initial function 2 ¡V Set Image buffer base address
//-----------------------------------------------------------
void IT8951SetImgBufBaseAddr(TDWord ulImgBufAddr)
{
  TWord usWordH = (TWord)((ulImgBufAddr >> 16) & 0x0000FFFF);
  TWord usWordL = (TWord)( ulImgBufAddr & 0x0000FFFF);
  //Write LISAR Reg
  IT8951WriteReg(LISAR + 2 ,usWordH);
  IT8951WriteReg(LISAR ,usWordL);
}

//-----------------------------------------------------
// 3.6. Display Functions
//-----------------------------------------------------

//-----------------------------------------------------------
//Display function 1 - Wait for LUT Engine Finish
//                     Polling Display Engine Ready by LUTNo
//-----------------------------------------------------------
void IT8951WaitForDisplayReady()
{
  //Check IT8951 Register LUTAFSR => NonZero ¡V Busy, 0 - Free
  while(IT8951ReadReg(LUTAFSR));
}

//-----------------------------------------------------------
//Display function 2 ¡V Load Image Area process
//-----------------------------------------------------------
void IT8951HostAreaPackedPixelWrite(IT8951LdImgInfo* pstLdImgInfo,IT8951AreaImgInfo* pstAreaImgInfo)
{
  TDWord i,j;
  //Source buffer address of Host
  TWord* pusFrameBuf = (TWord*)pstLdImgInfo->ulStartFBAddr;

  //Set Image buffer(IT8951) Base address
  IT8951SetImgBufBaseAddr(pstLdImgInfo->ulImgBufBaseAddr);
  //Send Load Image start Cmd
  IT8951LoadImgAreaStart(pstLdImgInfo , pstAreaImgInfo);
  Serial.printf("IT8951HostAreaPackedPixelWrite01\r\n");
  //Host Write Data
  for(j=0;j< pstAreaImgInfo->usHeight;j++)
  {
     for(i=0;i< pstAreaImgInfo->usWidth/2;i++)
      {
          //Write a Word(2-Bytes) for each time
          LCDWriteData(*pusFrameBuf);
          pusFrameBuf++;
      }
  }
  Serial.printf("IT8951HostAreaPackedPixelWrite02\r\n");
  //Send Load Img End Command
  IT8951LoadImgEnd();
}

//-----------------------------------------------------------
//Display function 2 ¡V Load Image Area process
//-----------------------------------------------------------
void IT8951HostAreaPackedPixelWriteFill(IT8951LdImgInfo* pstLdImgInfo,IT8951AreaImgInfo* pstAreaImgInfo, TWord pixel)
{
  TDWord i,j;
  //Source buffer address of Host
  //TWord* pusFrameBuf = (TWord*)pstLdImgInfo->ulStartFBAddr;

  //Set Image buffer(IT8951) Base address
  IT8951SetImgBufBaseAddr(pstLdImgInfo->ulImgBufBaseAddr);
  //Send Load Image start Cmd
  IT8951LoadImgAreaStart(pstLdImgInfo , pstAreaImgInfo);
  Serial.printf("IT8951HostAreaPackedPixelWriteFill01\r\n");
  //Host Write Data
  Serial.print("pstAreaImgInfo->usHeight  = ");
  Serial.println(pstAreaImgInfo->usHeight);
  Serial.print("pstAreaImgInfo->usWidth/2 = ");
  Serial.println(pstAreaImgInfo->usWidth/2);
  for(j=0;j< pstAreaImgInfo->usHeight;j++)
  {
    if((j % 100) == 0) {
      Serial.print("sending line ");
      Serial.print(j);
      Serial.print(" of ");
      Serial.println(pstAreaImgInfo->usHeight);
    }
    for(i=0;i< pstAreaImgInfo->usWidth/2;i++)
    {
      //Write a Word(2-Bytes) for each time
      LCDWriteData(pixel);
    }
  }
  Serial.printf("IT8951HostAreaPackedPixelWriteFill02\r\n");
  //Send Load Img End Command
  IT8951LoadImgEnd();
}

//-----------------------------------------------------------
//Display functions 3 - Application for Display panel Area
//-----------------------------------------------------------
void IT8951DisplayArea(TWord usX, TWord usY, TWord usW, TWord usH, TWord usDpyMode)
{
  //Send I80 Display Command (User defined command of IT8951)
  LCDWriteCmdCode(USDEF_I80_CMD_DPY_AREA); //0x0034
  //Write arguments
  LCDWriteData(usX);
  LCDWriteData(usY);
  LCDWriteData(usW);
  LCDWriteData(usH);
  LCDWriteData(usDpyMode);
}

//Display Area with bitmap on EPD
//-----------------------------------------------------------
// Display Function 4 - for Display Area for 1-bpp mode format
//   the bitmap(1bpp) mode will be enable when Display
//   and restore to Default setting (disable) after displaying finished
//-----------------------------------------------------------
void IT8951DisplayArea1bpp(TWord usX, TWord usY, TWord usW, TWord usH, TWord usDpyMode, TByte ucBGGrayVal, TByte ucFGGrayVal)
{
    //Set Display mode to 1 bpp mode - Set 0x18001138 Bit[18](0x1800113A Bit[2])to 1
    IT8951WriteReg(UP1SR+2, IT8951ReadReg(UP1SR+2) | (1<<2));
    
    //Set BitMap color table 0 and 1 , => Set Register[0x18001250]:
    //Bit[7:0]: ForeGround Color(G0~G15)  for 1
    //Bit[15:8]:Background Color(G0~G15)  for 0
    IT8951WriteReg(BGVR, (ucBGGrayVal<<8) | ucFGGrayVal);
    
    //Display
    IT8951DisplayArea( usX, usY, usW, usH, usDpyMode);
    IT8951WaitForDisplayReady();
    
    //Restore to normal mode
    IT8951WriteReg(UP1SR+2, IT8951ReadReg(UP1SR+2) & ~(1<<2));
}

//-------------------------------------------------------------------------------------------------------------
//  Command - 0x0037 for Display Base addr by User 
//  TDWord ulDpyBufAddr - Host programmer need to indicate the Image buffer address of IT8951
//                                         In current case, there is only one image buffer in IT8951 so far.
//                                         So Please set the Image buffer address you got  in initial stage.
//                                         (gulImgBufAddr by Get device information 0x0302 command)
//
//-------------------------------------------------------------------------------------------------------------
void IT8951DisplayAreaBuf(TWord usX, TWord usY, TWord usW, TWord usH, TWord usDpyMode, TDWord ulDpyBufAddr)
{
    //Send I80 Display Command (User defined command of IT8951)
    LCDWriteCmdCode(USDEF_I80_CMD_DPY_BUF_AREA); //0x0037
    
    //Write arguments
    LCDWriteData(usX);
    LCDWriteData(usY);
    LCDWriteData(usW);
    LCDWriteData(usH);
    LCDWriteData(usDpyMode);
    LCDWriteData((TWord)ulDpyBufAddr);       //Display Buffer Base address[15:0]
    LCDWriteData((TWord)(ulDpyBufAddr>>16)); //Display Buffer Base address[26:16]
}

//-----------------------------------------------------------
//Test function 1 ¡VSoftware Initial flow for testing
//-----------------------------------------------------------
void HostInit()
{
  //Get Device Info
  GetIT8951SystemInfo(&gstI80DevInfo);
  //Host Frame Buffer allocation
  gpFrameBuf = (TByte*)0xD0000000;
  //Get Image Buffer Address of IT8951
  gulImgBufAddr = gstI80DevInfo.usImgBufAddrL | (gstI80DevInfo.usImgBufAddrH << 16);
  
  //Set to Enable I80 Packed mode
  IT8951WriteReg(I80CPCR, 0x0001);
}

void IT8951DisplayImageClear(void)
{
  IT8951LdImgInfo stLdImgInfo;
  IT8951AreaImgInfo stAreaImgInfo;
  
  //Host Initial
  HostInit(); //Test Function 1
  
  //Check TCon is free ? Wait TCon Ready (optional)
  IT8951WaitForDisplayReady();

  //--------------------------------------------------------------------------------------------
  //      initial display - Display white only
  //--------------------------------------------------------------------------------------------
  //Load Image and Display
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)0;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP;
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
  stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
  
  //Display Area ¡V (x,y,w,h) with mode 0 for initial White to clear Panel
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 0);
}

void IT8951DisplayImageBmp(char *filename, uint8_t optionRotate, bool optionFlip)
{
  IT8951LdImgInfo stLdImgInfo;
  IT8951AreaImgInfo stAreaImgInfo;



  File     bmpFile;
  int      bmpWidth, bmpHeight;   // W+H in pixels
  uint8_t  bmpDepth;              // Bit depth (currently must be 24)
  uint32_t bmpImageoffset;        // Start of image data in file
  uint32_t rowSize;               // Not always = bmpWidth; may have padding
  uint8_t  sdbuffer[3*BUFFPIXEL]; // pixel buffer (R+G+B per pixel)
  uint8_t  buffidx = sizeof(sdbuffer); // Current position in sdbuffer
  boolean  goodBmp = false;       // Set to true on valid header parse
  boolean  flip    = true;        // BMP is stored bottom-to-top
  int      w, h, row, col;
  uint8_t  r, g, b;
  uint32_t pos = 0, startTime = millis();

  Serial.println();
  Serial.print(F("Loading image '"));
  Serial.print(filename);
  Serial.println('\'');

  // Open requested file on SD card
  if ((bmpFile = SD.open(filename)) == NULL) {
    Serial.print(F("File not found"));
    return;
  }

  // Parse BMP header
  if(read16(bmpFile) == 0x4D42) { // BMP signature
    Serial.print(F("File size    : ")); Serial.println(read32(bmpFile));
    (void)read32(bmpFile); // Read & ignore creator bytes
    bmpImageoffset = read32(bmpFile); // Start of image data
    Serial.print(F("Image Offset : ")); Serial.println(bmpImageoffset, DEC);
    // Read DIB header
    Serial.print(F("Header size  : ")); Serial.println(read32(bmpFile));
    bmpWidth  = read32(bmpFile);
    bmpHeight = read32(bmpFile);
    if(read16(bmpFile) == 1) { // # planes -- must be '1'
      bmpDepth = read16(bmpFile); // bits per pixel
      Serial.print(F("Bit Depth    : ")); Serial.println(bmpDepth);
      if((bmpDepth == 8) && (read32(bmpFile) == 0)) { // 0 = uncompressed

        goodBmp = true; // Supported BMP format -- proceed!
        Serial.print(F("Image size   : "));
        Serial.print(bmpWidth);
        Serial.print('x');
        Serial.println(bmpHeight);

        // BMP rows are padded (if needed) to 4-byte boundary
        rowSize = (bmpWidth * 3 + 3) & ~3;

        // If bmpHeight is negative, image is in top-down order.
        // This is not canon but has been observed in the wild.
        if(bmpHeight < 0) {
          bmpHeight = -bmpHeight;
          flip      = false;
        }

  
        //Host Initial
        HostInit(); //Test Function 1
        
        //Check TCon is free ? Wait TCon Ready (optional)
        IT8951WaitForDisplayReady();
        
        //Setting Load image information
        stLdImgInfo.ulStartFBAddr    = (TDWord)0;
        stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
        stLdImgInfo.usPixelFormat    = IT8951_8BPP; 
        stLdImgInfo.usRotate         = (TWord)optionRotate;
        stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
        //Set Load Area
        stAreaImgInfo.usX      = 0;
        stAreaImgInfo.usY      = 0;
        stAreaImgInfo.usWidth  = 1200;
        stAreaImgInfo.usHeight = 825;
      
      
      
        //Load Image from Host to IT8951 Image Buffer
        //Set Image buffer(IT8951) Base address
        IT8951SetImgBufBaseAddr(stLdImgInfo.ulImgBufBaseAddr);
        //Send Load Image start Cmd
        IT8951LoadImgAreaStart(&stLdImgInfo, &stAreaImgInfo);
        Serial.printf("IT8951HostAreaPackedPixelWrite01\r\n");
        //Host Write Data
      
        bmpFile.seek(bmpImageoffset);
        uint16_t tempData;
        unsigned char lineBuffer[1200];
        uint16_t lineBufferPos = 0;
      
        uint32_t chunk_size = 1200 * 54;
        uint32_t file_size = stAreaImgInfo.usHeight * stAreaImgInfo.usWidth;
        uint32_t chunk_total = file_size / chunk_size + 1;
      
        Serial.print("chunk_size  = ");
        Serial.println(chunk_size);
        Serial.print("chunk_total = ");
        Serial.println(chunk_total);

        //read in first chunk
        for(uint32_t chunk_current = 0; chunk_current < chunk_total; chunk_current++) {
          if(chunk_current < (chunk_total - 1)) {
            Serial.print("reading in chunk ");
            Serial.print(chunk_current);
            Serial.print(" of ");
            Serial.print(chunk_total);
            Serial.print(" chunk size = ");
            Serial.println(chunk_size);
            bmpFile.read(transferBuffer, chunk_size);
            if(optionFlip == false) {
              for(uint32_t i = 0; i < chunk_size / 2; i++) {
                tempData = (transferBuffer[(i * 2) + 1] << 8);
                tempData |= (transferBuffer[(i * 2)]);
                LCDWriteData(tempData);
              }
            } else {
              lineBufferPos = 1200;
              for(uint16_t i = 0; i < chunk_size; i++) {
                lineBuffer[lineBufferPos--] = transferBuffer[i];
                if(lineBufferPos == 0) {
                  for(uint16_t j = 0; j < 1200 / 2; j++) {
                    tempData = (lineBuffer[(j * 2) + 1] << 8);
                    tempData |= (lineBuffer[(j * 2)]);
                    LCDWriteData(tempData);
                  }
                  lineBufferPos = 1200;
                }
              }
            }
          } else {
            uint32_t remainder_size = file_size - (chunk_size * (chunk_total - 1));
            Serial.print("reading in chunk ");
            Serial.print(chunk_current);
            Serial.print(" of ");
            Serial.print(chunk_total);
            Serial.print(" (last) remainder_size size = ");
            Serial.println(remainder_size);
            bmpFile.read(transferBuffer, remainder_size);
            if(optionFlip == false) {
              for(uint32_t i = 0; i < remainder_size / 2; i++) {
                tempData = (transferBuffer[(i * 2) + 1] << 8);
                tempData |= (transferBuffer[(i * 2)]);
                LCDWriteData(tempData);
              }
            } else {
              lineBufferPos = 1200;
              for(uint16_t i = 0; i < remainder_size; i++) {
                lineBuffer[lineBufferPos--] = transferBuffer[i];
                if(lineBufferPos == 0) {
                  for(uint16_t j = 0; j < 1200 / 2; j++) {
                    tempData = (lineBuffer[(j * 2) + 1] << 8);
                    tempData |= (lineBuffer[(j * 2)]);
                    LCDWriteData(tempData);
                  }
                  lineBufferPos = 1200;
                }
              }
            }
          }
        }
        
        Serial.printf("IT8951HostAreaPackedPixelWrite02\r\n");
        //Send Load Img End Command
        IT8951LoadImgEnd();
      
        //Display Area ¡V (x,y,w,h) with mode 0 for initial White to clear Panel
        IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 0);
        //Display Area ¡V (x,y,w,h) with mode 2 for fast gray clear mode - depends on current waveform 
        IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 2);
      }
    }
  }
  bmpFile.close();
  if(!goodBmp) Serial.println(F("BMP format not recognized."));
}

void IT8951TestImage(uint16_t pixel)
{
  IT8951LdImgInfo stLdImgInfo;
  IT8951AreaImgInfo stAreaImgInfo;
  
  //Host Initial
  HostInit(); //Test Function 1
  //Prepare image
  //Write pixel 0xF0(White) to Frame Buffer
  //memset(gpFrameBuf, 0xF0, gstI80DevInfo.usPanelW * gstI80DevInfo.usPanelH);
  
  //Check TCon is free ? Wait TCon Ready (optional)
  IT8951WaitForDisplayReady();
  
  //--------------------------------------------------------------------------------------------
  //      initial display - Display white only
  //--------------------------------------------------------------------------------------------
  //Load Image and Display
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)0;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP;
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
  stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
  
  Serial.printf("IT8951DisplayExample 01\r\n");
  //Load Image from Host to IT8951 Image Buffer
  IT8951HostAreaPackedPixelWriteFill(&stLdImgInfo, &stAreaImgInfo, pixel);//Display function 2
  Serial.printf("IT8951DisplayExample 02\r\n");
  //Display Area ¡V (x,y,w,h) with mode 0 for initial White to clear Panel
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 0);
  Serial.printf("IT8951DisplayExample 03\r\n");
  //Display Area ¡V (x,y,w,h) with mode 2 for fast gray clear mode - depends on current waveform 
  IT8951DisplayArea(0,0, stAreaImgInfo.usWidth, stAreaImgInfo.usHeight, 2);
  Serial.printf("IT8951DisplayExample 04\r\n");
}

//-----------------------------------------------------------
//Test function 2 ¡V Example of Display Flow
//-----------------------------------------------------------
void IT8951DisplayExample()
{
  IT8951LdImgInfo stLdImgInfo;
  IT8951AreaImgInfo stAreaImgInfo;
  
  //Host Initial
  HostInit(); //Test Function 1
  //Prepare image
  //Write pixel 0xF0(White) to Frame Buffer
  memset(gpFrameBuf, 0xF0, gstI80DevInfo.usPanelW * gstI80DevInfo.usPanelH);
  
  //Check TCon is free ? Wait TCon Ready (optional)
  IT8951WaitForDisplayReady();
  
  //--------------------------------------------------------------------------------------------
  //      initial display - Display white only
  //--------------------------------------------------------------------------------------------
  //Load Image and Display
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)gpFrameBuf;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP;
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
  stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
  
  Serial.printf("IT8951DisplayExample 01\r\n");
  //Load Image from Host to IT8951 Image Buffer
  IT8951HostAreaPackedPixelWrite(&stLdImgInfo, &stAreaImgInfo);//Display function 2
  Serial.printf("IT8951DisplayExample 02\r\n");
  //Display Area ¡V (x,y,w,h) with mode 0 for initial White to clear Panel
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 0);
  Serial.printf("IT8951DisplayExample 03\r\n");
  //--------------------------------------------------------------------------------------------
  //      Regular display - Display Any Gray colors with Mode 2 or others
  //--------------------------------------------------------------------------------------------
  //Preparing buffer to All black (8 bpp image)
  //or you can create your image pattern here..
  memset(gpFrameBuf, 0x00, gstI80DevInfo.usPanelW * gstI80DevInfo.usPanelH);
   
  IT8951WaitForDisplayReady();
  
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)gpFrameBuf;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP; 
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
  stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
  
  //Load Image from Host to IT8951 Image Buffer
  IT8951HostAreaPackedPixelWrite(&stLdImgInfo, &stAreaImgInfo);//Display function 2
  //Display Area ¡V (x,y,w,h) with mode 2 for fast gray clear mode - depends on current waveform 
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 2);
}

//0x00ÊÇºÚ£¬0xf0ÊÇ°×
void IT8951DisplayExample2()
{
  IT8951LdImgInfo stLdImgInfo;
  IT8951AreaImgInfo stAreaImgInfo;
  
  //--------------------------------------------------------------------------------------------
  //      Regular display - Display Any Gray colors with Mode 2 or others
  //--------------------------------------------------------------------------------------------
  //Preparing buffer to All black (8 bpp image)
  //or you can create your image pattern here..
//memset(gpFrameBuf, 0x00, gstI80DevInfo.usPanelW * gstI80DevInfo.usPanelH);
  memset(gpFrameBuf              , 0x00, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 1, 0x11, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 2, 0x22, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 3, 0x33, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 4, 0x44, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 5, 0x55, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 6, 0x66, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 7, 0x77, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 8, 0x88, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 9, 0x99, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 10, 0xaa, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 11, 0xbb, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 12, 0xcc, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 13, 0xdd, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 14, 0xee, 1200 * 51 * 1);
  memset(gpFrameBuf+1200 * 51 * 15, 0xff, 1200 * 51 * 2);

  IT8951WaitForDisplayReady();
  
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)gpFrameBuf;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP; 
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
  stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
  
  //Load Image from Host to IT8951 Image Buffer
  IT8951HostAreaPackedPixelWrite(&stLdImgInfo, &stAreaImgInfo);//Display function 2
  //Display Area ¡V (x,y,w,h) with mode 2 for fast gray clear mode - depends on current waveform 
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 2);
}

extern const unsigned char pic[];
void IT8951DisplayExample3()
{
  IT8951LdImgInfo stLdImgInfo;
  IT8951AreaImgInfo stAreaImgInfo;
  TDWord i;
  
  for (i = 0;i < 1200*825;i++)
  {
    gpFrameBuf[i] = pic[i];
  }

  IT8951WaitForDisplayReady();
  
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)gpFrameBuf;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP; 
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = 1200;
  stAreaImgInfo.usHeight = 825;
  
  //Load Image from Host to IT8951 Image Buffer
  IT8951HostAreaPackedPixelWrite(&stLdImgInfo, &stAreaImgInfo);//Display function 2
  //Display Area ¡V (x,y,w,h) with mode 2 for fast gray clear mode - depends on current waveform 
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 2);
}

//-----------------------------------------------------------
// Load 1bpp image flow (must display with IT8951DisplayArea1bpp()
//-----------------------------------------------------------

void IT8951Load1bppImage(TByte* p1bppImgBuf, TWord usX, TWord usY, TWord usW, TWord usH)
{
    IT8951LdImgInfo stLdImgInfo;
    IT8951AreaImgInfo stAreaImgInfo;
  
    //Setting Load image information
    stLdImgInfo.ulStartFBAddr    = (TDWord) p1bppImgBuf;
    stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
    stLdImgInfo.usPixelFormat    = IT8951_8BPP; //we use 8bpp because IT8951 dose not support 1bpp mode for load image¡Aso we use Load 8bpp mode ,but the transfer size needs to be reduced to Size/8
    stLdImgInfo.usRotate         = IT8951_ROTATE_0;
    stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
    //Set Load Area
    stAreaImgInfo.usX      = usX/8;
    stAreaImgInfo.usY      = usY;
    stAreaImgInfo.usWidth  = usW/8;//1bpp, Chaning Transfer size setting to 1/8X of 8bpp mode 
    stAreaImgInfo.usHeight = usH;
    Serial.printf("IT8951HostAreaPackedPixelWrite [wait]\n\r");
    //Load Image from Host to IT8951 Image Buffer
    IT8951HostAreaPackedPixelWrite(&stLdImgInfo, &stAreaImgInfo);//Display function 2
}

//-----------------------------------------------------------
//Test function 3 - Example of Display 1bpp Flow
//-----------------------------------------------------------
void IT8951Display1bppExample()
{
    IT8951AreaImgInfo stAreaImgInfo;
    
    //Host Initial
    HostInit(); //Test Function 1
    //Prepare image
    //Write pixel 0x00(Black) to Frame Buffer
    //or you can create your image pattern here..
    memset(gpFrameBuf, 0x00, (gstI80DevInfo.usPanelW * gstI80DevInfo.usPanelH)/8);//Host Frame Buffer(Source)
    
    //Check TCon is free ? Wait TCon Ready (optional)
    IT8951WaitForDisplayReady();
    
    //Load Image and Display
    //Set Load Area
    stAreaImgInfo.usX      = 0;
    stAreaImgInfo.usY      = 0;
    stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
    stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
    //Load Image from Host to IT8951 Image Buffer
    IT8951Load1bppImage(gpFrameBuf, stAreaImgInfo.usX, stAreaImgInfo.usY, stAreaImgInfo.usWidth, stAreaImgInfo.usHeight);//Display function 4, Arg
    
    //Display Area - (x,y,w,h) with mode 2 for Gray Scale
    //e.g. if we want to set b0(Background color) for Black-0x00 , Set b1(Foreground) for White-0xFF
    IT8951DisplayArea1bpp(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 0, 0x00, 0xFF);
}

void IT8951Display1bppExample2()
{
   IT8951AreaImgInfo stAreaImgInfo;
    
    //Host Initial
    HostInit(); //Test Function 1
    //Prepare image
    //Write pixel 0x00(Black) to Frame Buffer
    //or you can create your image pattern here..
    memset(gpFrameBuf, 0xff, (gstI80DevInfo.usPanelW * gstI80DevInfo.usPanelH)/8);//Host Frame Buffer(Source)
    
    //Check TCon is free ? Wait TCon Ready (optional)
    IT8951WaitForDisplayReady();
    
    //Load Image and Display
    //Set Load Area
    stAreaImgInfo.usX      = 0;
    stAreaImgInfo.usY      = 0;
    stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
    stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
    //Load Image from Host to IT8951 Image Buffer
    IT8951Load1bppImage(gpFrameBuf, stAreaImgInfo.usX, stAreaImgInfo.usY, stAreaImgInfo.usWidth, stAreaImgInfo.usHeight);//Display function 4, Arg
    
    //Display Area - (x,y,w,h) with mode 2 for Gray Scale
    //e.g. if we want to set b0(Background color) for Black-0x00 , Set b1(Foreground) for White-0xFF
    IT8951DisplayArea1bpp(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 0, 0x00, 0xFF);
}

//-----------------------------------------------------------
//Test function 2 ¡V Example of Display Flow
//-----------------------------------------------------------
void IT8951DisplayClear()
{
  IT8951LdImgInfo stLdImgInfo;
  IT8951AreaImgInfo stAreaImgInfo;
  
  
  //--------------------------------------------------------------------------------------------
  //      Regular display - Display Any Gray colors with Mode 2 or others
  //--------------------------------------------------------------------------------------------
  //Preparing buffer to All black (8 bpp image)
  //or you can create your image pattern here..
  memset(gpFrameBuf, 0x00, gstI80DevInfo.usPanelW * gstI80DevInfo.usPanelH);
   
  IT8951WaitForDisplayReady();
  
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)gpFrameBuf;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP; 
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
  stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
  
  //Load Image from Host to IT8951 Image Buffer
  IT8951HostAreaPackedPixelWrite(&stLdImgInfo, &stAreaImgInfo);//Display function 2
  //Display Area ¡V (x,y,w,h) with mode 2 for fast gray clear mode - depends on current waveform 
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 2);
  
  
  //--------------------------------------------------------------------------------------------
  //      initial display - Display white only
  //--------------------------------------------------------------------------------------------
  //Load Image and Display
  //Setting Load image information
  stLdImgInfo.ulStartFBAddr    = (TDWord)gpFrameBuf;
  stLdImgInfo.usEndianType     = IT8951_LDIMG_L_ENDIAN;
  stLdImgInfo.usPixelFormat    = IT8951_8BPP;
  stLdImgInfo.usRotate         = IT8951_ROTATE_0;
  stLdImgInfo.ulImgBufBaseAddr = gulImgBufAddr;
  //Set Load Area
  stAreaImgInfo.usX      = 0;
  stAreaImgInfo.usY      = 0;
  stAreaImgInfo.usWidth  = gstI80DevInfo.usPanelW;
  stAreaImgInfo.usHeight = gstI80DevInfo.usPanelH;
  
  //Load Image from Host to IT8951 Image Buffer
  IT8951HostAreaPackedPixelWrite(&stLdImgInfo, &stAreaImgInfo);//Display function 2
  //Display Area ¡V (x,y,w,h) with mode 0 for initial White to clear Panel
  IT8951DisplayArea(0,0, gstI80DevInfo.usPanelW, gstI80DevInfo.usPanelH, 0);
}
