/*******************************************
* Author: Qingfeng Xia  @ iesensor.com
* BSD 3 claused Licensed  2013
* https://www.iesensor.com/blog/2013/07/09/software-spi-library-for-arduino-tested-on-mega2560/
*********************************************/

#include "software_spi.h"
#include "Arduino.h"
//#include "adc_board.h"

#if USE_SOFTWARE_SPI
  static inline void delayCycles(int cycles)
  {
    for (int i=0; i++; i<cycles)
      __asm__("nop\n\t");
      // __nop();
  }
  
  
  #if SPI_MSBFIRST
    //unsigned char msk[] = {0x80, 0x40, 0x20, 0x10, 0x8, 0x4, 0x2, 0x1};
    //#define PCK() (bits[0] << 7 | bits[1] << 6 | bits[2] << 5 | bits[3] << 4 | bits[4] << 3 | bits[5] << 2 | bits[6] << 1 | bits[7])
    uint16_t msk[] = {0x8000, 0x4000, 0x2000, 0x1000, 0x800, 0x400, 0x200, 0x100, 0x80, 0x40, 0x20, 0x10, 0x8, 0x4, 0x2, 0x1};
    #define PCK() (bits[0] << 15 | bits[1] << 14 | bits[2] << 13 | bits[3] << 12 | bits[4] << 11 | bits[5] << 10 | bits[6] << 9 | bits[7] << 8 | bits[8] << 7 | bits[9] << 6 | bits[10] << 5 | bits[11] << 4 | bits[12] << 3 | bits[13] << 2 | bits[14] << 1 | bits[15])
  #else
    //unsigned char msk[] = {0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80};
    //#define PCK() (bits[0]  | bits[1] << 1 | bits[2] << 2 | bits[3] << 3 | bits[4] << 4 | bits[5] << 5 | bits[6] << 6 | bits[7]<<7)
    uint16_t msk[] = {0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80, 0x100, 0x200, 0x400, 0x800, 0x1000, 0x2000, 0x4000, 0x8000};
    #define PCK() (bits[0] << 0 | bits[1] << 1 | bits[2] << 2 | bits[3] << 3 | bits[4] << 4 | bits[5] << 5 | bits[6] << 6 | bits[7] << 7 | bits[8] << 8 | bits[9] << 9 | bits[10] << 10 | bits[11] << 11 | bits[12] << 12 | bits[13] << 13 | bits[14] << 14 | bits[15] << 15)
  #endif
  
  
  
  void spi_setup()
  {
    pinMode(_SS, OUTPUT);
    pinMode(_MISO, INPUT);
    pinMode(_MOSI, OUTPUT);
    pinMode(_SCK, OUTPUT);
    //
     digitalWrite(_SS, HIGH);
    #if _CPOL
     digitalWrite(_SCK, HIGH);
    #else
     digitalWrite(_SCK, LOW);
    #endif
  }
  
  // chip selection has been declared outside
  inline void spi_select() {digitalWrite(_SS, LOW);}
  inline void spi_unselect() {digitalWrite(_SS, HIGH);}
  
  //mode 0: SCK idle low, phase: reading at middle of SCK HIGH pulse
  //mode 1: SCK idle low, phase: reading at middle of SCK LOW pulse
  //this big-bang should work for both  CPHA=1  and CPHA=0
  //uint8_t spi_transfer(uint8_t b) 
  uint16_t spi_transfer(uint16_t b) 
  {
    //uint8_t reply=0;
    uint16_t reply=0;
    char bits[16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  //reading buffer
  
    /*  hardware SPI
    SPDR=b;
    spi_wait();
    reply = SPDR;
    return reply;*/
  
    //cli();  it will cause error on Arduino, most of SPI should be interrupt tolerable
    //spi_select(); // should be called outside, may be required by one transition
    delayCycles(DELAY_CYCLES);   // checking timing characteristics, need delay from CS to rising edge?
    // here, delay is added, to make CPHA=1 and CPHA=0 both work!
    
    //for(uint8_t _bit = 0;_bit < 8;_bit++)
    for(uint8_t _bit = 0;_bit < 16;_bit++)
    {
    #if _CPHA
      #if _CPOL 
        digitalWrite(_SCK, LOW);  //  propagation at rising edge
      #else
        digitalWrite(_SCK, HIGH);  // change this to LOW for CPOL=1
      #endif

      digitalWrite(_SCK, HIGH);  // change this to LOW for CPOL=1
      delayCycles(DELAY_CYCLES_P0);    
  
      digitalWrite(_MOSI, !!(b & msk[_bit]));
      delayCycles(DELAY_CYCLES_P1); //  propagation

      #if _CPOL 
        digitalWrite(_SCK, HIGH);  // data will be captured at falling edge
      #else
        digitalWrite(_SCK, LOW);  
      #endif

      delayCycles(DELAY_CYCLES_C0); // holding low, so there is enough time for data preparation and changing

      bits[_bit] = digitalRead(_MISO); // reading at the middle of SCK pulse
      delayCycles(DELAY_CYCLES_C1);  // wait until data is fetched by slave device,  while SCK low, checking DATAsheet for this interval 

    #else
      // changing MOSI big while SCK low, propogation 
      digitalWrite(_MOSI, !!(b & msk[_bit]));
      delayCycles(DELAY_CYCLES_P1); // there is a requirement of LOW and HIGH have identical interval!

      #if _CPOL
        digitalWrite(_SCK, LOW); 
      #else
        digitalWrite(_SCK, HIGH);
      #endif
       delayCycles(DELAY_CYCLES_C0);    // 
    
      bits[_bit] = digitalRead(_MISO); // reading at the middle of SCK pulse
      delayCycles(DELAY_CYCLES_C1);  // wait until data is fetched by slave device,  while SCK high, checking DATAsheet for this interval 
       
      #if _CPOL
        digitalWrite(_SCK, HIGH);
      #else
        digitalWrite(_SCK, LOW);  // data will change at falling edge
      #endif
        delayCycles(DELAY_CYCLES_P0); // holding low, so there is enough time for data preparation and changing
    #endif
    }
    delayCycles(DELAY_CYCLES);  // checking timing characteristics, it is no needed by AD7730, from CS to rising edge
    // spi_unselect();  

    //sei();  // it will cause error on Arduino, most of SPI should be interrupt tolerable

    reply = PCK();
    
    return reply;
  }
#endif

#ifdef USE_HARDWARE_SPI_DEFAULT
  #include <SPI.h>
  #include "wiring_private.h" // pinPeripheral() function

  void spi_setup() {
    // do this first, for Reasons
    SPI.begin();
    //mode 0: SCK idle low, phase: reading at middle of SCK HIGH pulse
    //mode 1: SCK idle low, phase: reading at middle of SCK LOW pulse
    //this big-bang should work for both  CPHA=1  and CPHA=0
    //24Mhz is max speed for IT8951, transfer errors so speed reduced to 12Mhz
    SPI.beginTransaction(SPISettings(12000000, MSBFIRST, SPI_MODE0));
  }

  uint16_t spi_transfer(uint16_t b) {
    return SPI.transfer16(b);
  }
#endif

#ifdef USE_HARDWARE_SPI_SERCOM6
  /*
  in  Arduino 'Pin' SERCOM    SERCOM alt
  -----------------------------------------
  PC04  D48   SERCOM6.0 none
  PC05  D49   SERCOM6.1 none
  PC06  D46   SERCOM6.2 none
  PC07  D47   SERCOM6.3 none
  

  SCK  on D49 (SERCOM6.1)
  MISO on D47 (SERCOM6.3)
  MOSI on D48 (SERCOM6.0)
  */

  #include <SPI.h>
  #include "wiring_private.h" // pinPeripheral() function

  SPIClass i8951_SPI (&sercom6, 49, 47, 48, SPI_PAD_0_SCK_1, SERCOM_RX_PAD_3);
  
  void spi_setup() {
    // do this first, for Reasons
    pinPeripheral(47, PIO_SERCOM);
    pinPeripheral(48, PIO_SERCOM);
    pinPeripheral(49, PIO_SERCOM);

    i8951_SPI.begin();
    
    //mode 0: SCK idle low, phase: reading at middle of SCK HIGH pulse
    //mode 1: SCK idle low, phase: reading at middle of SCK LOW pulse
    //this big-bang should work for both  CPHA=1  and CPHA=0
    //24Mhz is max speed for IT8951, transfer errors so speed reduced to 12Mhz
    i8951_SPI.beginTransaction(SPISettings(12000000, MSBFIRST, SPI_MODE0));
  }

  uint16_t spi_transfer(uint16_t b) {
    return i8951_SPI.transfer16(b);
  }
#endif
